@@ -13,7 +13,7 @@
 #include "mozilla/IntegerPrintfMacros.h"
 #include "mozilla/Preferences.h"
 #include "mozilla/Services.h"
-#include "mozilla/unused.h"
+#include "mozilla/Unused.h"
 #include "AudioChannelService.h"
 #include "mozilla/Logging.h"
 #include "nsPrintfCString.h"
@@ -344,7 +344,6 @@ public:
 private:
   static uint32_t sBackgroundPerceivableGracePeriodMS;
   static uint32_t sBackgroundGracePeriodMS;
-  static uint32_t sMemoryPressureGracePeriodMS;
 
   void FireTestOnlyObserverNotification(
     const char* aTopic,
@@ -369,7 +368,6 @@ private:
   nsAutoCString mNameWithComma;
 
   nsCOMPtr<nsITimer> mResetPriorityTimer;
-  nsCOMPtr<nsITimer> mMemoryPressureTimer;
 };
 
 /* static */ bool ProcessPriorityManagerImpl::sInitialized = false;
@@ -382,7 +380,6 @@ private:
   ProcessPriorityManagerImpl::sSingleton;
 /* static */ uint32_t ParticularProcessPriorityManager::sBackgroundPerceivableGracePeriodMS = 0;
 /* static */ uint32_t ParticularProcessPriorityManager::sBackgroundGracePeriodMS = 0;
-/* static */ uint32_t ParticularProcessPriorityManager::sMemoryPressureGracePeriodMS = 0;
 
 NS_IMPL_ISUPPORTS(ProcessPriorityManagerImpl,
                   nsIObserver,
@@ -529,13 +526,6 @@ already_AddRefed<ParticularProcessPriorityManager>
 ProcessPriorityManagerImpl::GetParticularProcessPriorityManager(
   ContentParent* aContentParent)
 {
-#ifdef MOZ_NUWA_PROCESS
-  // Do not attempt to change the priority of the Nuwa process
-  if (aContentParent->IsNuwaProcess()) {
-    return nullptr;
-  }
-#endif
-
   RefPtr<ParticularProcessPriorityManager> pppm;
   uint64_t cpId = aContentParent->ChildID();
   mParticularManagers.Get(cpId, &pppm);
@@ -702,8 +692,6 @@ ParticularProcessPriorityManager::StaticInit()
                                "dom.ipc.processPriorityManager.backgroundPerceivableGracePeriodMS");
   Preferences::AddUintVarCache(&sBackgroundGracePeriodMS,
                                "dom.ipc.processPriorityManager.backgroundGracePeriodMS");
-  Preferences::AddUintVarCache(&sMemoryPressureGracePeriodMS,
-                               "dom.ipc.processPriorityManager.memoryPressureGracePeriodMS");
 }
 
 void
@@ -874,9 +862,9 @@ ParticularProcessPriorityManager::OnRemoteBrowserFrameShown(nsISupports* aSubjec
   }
 
   // Ignore notifications that aren't from a BrowserOrApp
-  bool isBrowserOrApp;
-  fl->GetOwnerIsBrowserOrAppFrame(&isBrowserOrApp);
-  if (isBrowserOrApp) {
+  bool isMozBrowserOrApp;
+  fl->GetOwnerIsMozBrowserOrAppFrame(&isMozBrowserOrApp);
+  if (isMozBrowserOrApp) {
     ResetPriority();
   }
 
@@ -1010,28 +998,17 @@ ParticularProcessPriorityManager::ScheduleResetPriority(TimeoutPref aTimeoutPref
   }
 
   LOGP("Scheduling reset timer to fire in %dms.", timeout);
-  mResetPriorityTimer = do_CreateInstance("@mozilla.org/timer;1");
+  mResetPriorityTimer = do_CreateInstance(NS_TIMER_CONTRACTID);
   mResetPriorityTimer->InitWithCallback(this, timeout, nsITimer::TYPE_ONE_SHOT);
 }
 
 NS_IMETHODIMP
 ParticularProcessPriorityManager::Notify(nsITimer* aTimer)
 {
-  if (mResetPriorityTimer == aTimer) {
-    LOGP("Reset priority timer callback; about to ResetPriorityNow.");
-    ResetPriorityNow();
-    mResetPriorityTimer = nullptr;
-    return NS_OK;
-  }
-
-  if (mContentParent && mMemoryPressureTimer == aTimer) {
-    Unused << mContentParent->SendFlushMemory(NS_LITERAL_STRING("lowering-priority"));
-    mMemoryPressureTimer = nullptr;
-    return NS_OK;
-  }
-
-  NS_WARNING("Unexpected timer!");
-  return NS_ERROR_INVALID_POINTER;
+  LOGP("Reset priority timer callback; about to ResetPriorityNow.");
+  ResetPriorityNow();
+  mResetPriorityTimer = nullptr;
+  return NS_OK;
 }
 
 bool
@@ -1061,10 +1038,6 @@ ParticularProcessPriorityManager::IsExpectingSystemMessage()
     if (!bf) {
       continue;
     }
-
-    if (bf->GetIsExpectingSystemMessage()) {
-      return true;
-    }
   }
 
   return false;
@@ -1156,18 +1129,6 @@ ParticularProcessPriorityManager::SetPriorityNow(ProcessPriority aPriority,
       NotifyProcessPriorityChanged(this, oldPriority);
 
     Unused << mContentParent->SendNotifyProcessPriorityChanged(mPriority);
-
-    if (mMemoryPressureTimer) {
-      mMemoryPressureTimer->Cancel();
-      mMemoryPressureTimer = nullptr;
-    }
-
-    if (aPriority < PROCESS_PRIORITY_FOREGROUND) {
-      mMemoryPressureTimer = do_CreateInstance(NS_TIMER_CONTRACTID);
-      mMemoryPressureTimer->InitWithCallback(this,
-                                             sMemoryPressureGracePeriodMS,
-                                             nsITimer::TYPE_ONE_SHOT);
-    }
   }
 
   FireTestOnlyObserverNotification("process-priority-set",
