@@ -7,9 +7,6 @@
 #ifdef XP_MACOSX
 #include "MacIOSurfaceTextureHostBasic.h"
 #endif
-#ifdef MOZ_WIDGET_GONK
-#include "GrallocTextureHostBasic.h"
-#endif
 
 using namespace mozilla::gl;
 using namespace mozilla::gfx;
@@ -28,13 +25,6 @@ CreateTextureHostBasic(const SurfaceDescriptor& aDesc,
       aDesc.get_SurfaceDescriptorMacIOSurface();
     return MakeAndAddRef<MacIOSurfaceTextureHostBasic>(aFlags, desc);
   }
-#endif
-#ifdef MOZ_WIDGET_GONK
-  if (aDesc.type() == SurfaceDescriptor::TSurfaceDescriptorGralloc) {
-      const SurfaceDescriptorGralloc& desc =
-        aDesc.get_SurfaceDescriptorGralloc();
-    return MakeAndAddRef<GrallocTextureHostBasic>(aFlags, desc);
-    }
 #endif
   return CreateBackendIndependentTextureHost(aDesc, aDeallocator, aFlags);
 }
