@@ -27,7 +27,6 @@
 #include "mozilla/layers/TextureClientPool.h"
 #include "ClientLayerManager.h"
 #include "mozilla/mozalloc.h"           // for operator delete
-#include "nsAutoPtr.h"                  // for nsRefPtr
 #include "nsISupportsImpl.h"            // for MOZ_COUNT_DTOR
 #include "nsPoint.h"                    // for nsIntPoint
 #include "nsRect.h"                     // for mozilla::gfx::IntRect
@@ -35,8 +34,6 @@
 #include "nsTArray.h"                   // for nsTArray, nsTArray_Impl, etc
 #include "nsExpirationTracker.h"
 #include "mozilla/layers/ISurfaceAllocator.h"
-#include "gfxReusableSurfaceWrapper.h"
-#include "pratom.h"                     // For PR_ATOMIC_INCREMENT/DECREMENT
 
 namespace mozilla {
 namespace layers {
@@ -44,105 +41,6 @@ namespace layers {
 class ClientTiledPaintedLayer;
 class ClientLayerManager;
 
-
-// A class to help implement copy-on-write semantics for shared tiles.
-class gfxSharedReadLock {
-protected:
-  virtual ~gfxSharedReadLock() {}
-
-public:
-  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(gfxSharedReadLock)
-
-  virtual int32_t ReadLock() = 0;
-  virtual int32_t ReadUnlock() = 0;
-  virtual int32_t GetReadCount() = 0;
-  virtual bool IsValid() const = 0;
-
-  enum gfxSharedReadLockType {
-    TYPE_MEMORY,
-    TYPE_SHMEM
-  };
-  virtual gfxSharedReadLockType GetType() = 0;
-
-protected:
-  NS_DECL_OWNINGTHREAD
-};
-
-class gfxMemorySharedReadLock : public gfxSharedReadLock {
-public:
-  gfxMemorySharedReadLock();
-
-protected:
-  ~gfxMemorySharedReadLock();
-
-public:
-  virtual int32_t ReadLock() override;
-
-  virtual int32_t ReadUnlock() override;
-
-  virtual int32_t GetReadCount() override;
-
-  virtual gfxSharedReadLockType GetType() override { return TYPE_MEMORY; }
-
-  virtual bool IsValid() const override { return true; };
-
-private:
-  int32_t mReadCount;
-};
-
-class gfxShmSharedReadLock : public gfxSharedReadLock {
-private:
-  struct ShmReadLockInfo {
-    int32_t readCount;
-  };
-
-public:
-  explicit gfxShmSharedReadLock(ISurfaceAllocator* aAllocator);
-
-protected:
-  ~gfxShmSharedReadLock();
-
-public:
-  virtual int32_t ReadLock() override;
-
-  virtual int32_t ReadUnlock() override;
-
-  virtual int32_t GetReadCount() override;
-
-  virtual bool IsValid() const override { return mAllocSuccess; };
-
-  virtual gfxSharedReadLockType GetType() override { return TYPE_SHMEM; }
-
-  mozilla::layers::ShmemSection& GetShmemSection() { return mShmemSection; }
-
-  static already_AddRefed<gfxShmSharedReadLock>
-  Open(mozilla::layers::ISurfaceAllocator* aAllocator, const mozilla::layers::ShmemSection& aShmemSection)
-  {
-    MOZ_RELEASE_ASSERT(aShmemSection.shmem().IsReadable());
-    RefPtr<gfxShmSharedReadLock> readLock = new gfxShmSharedReadLock(aAllocator, aShmemSection);
-    return readLock.forget();
-  }
-
-private:
-  gfxShmSharedReadLock(ISurfaceAllocator* aAllocator, const mozilla::layers::ShmemSection& aShmemSection)
-    : mAllocator(aAllocator)
-    , mShmemSection(aShmemSection)
-    , mAllocSuccess(true)
-  {
-    MOZ_COUNT_CTOR(gfxShmSharedReadLock);
-  }
-
-  ShmReadLockInfo* GetShmReadLockInfoPtr()
-  {
-    return reinterpret_cast<ShmReadLockInfo*>
-      (mShmemSection.shmem().get<char>() + mShmemSection.offset());
-  }
-
-  RefPtr<ISurfaceAllocator> mAllocator;
-  mozilla::layers::ShmemSection mShmemSection;
-  bool mAllocSuccess;
-};
-
 /**
  * Represent a single tile in tiled buffer. The buffer keeps tiles,
  * each tile keeps a reference to a texture client and a read-lock. This
@@ -172,41 +70,16 @@ struct TileClient
     return mFrontBuffer != o.mFrontBuffer;
   }
 
-  void SetLayerManager(ClientLayerManager *aManager)
-  {
-    mManager = aManager;
-  }
   void SetTextureAllocator(TextureClientAllocator* aAllocator)
   {
     mAllocator = aAllocator;
   }
 
-  void SetCompositableClient(CompositableClient* aCompositableClient)
-  {
-    mCompositableClient = aCompositableClient;
-  }
-
   bool IsPlaceholderTile() const
   {
     return mBackBuffer == nullptr && mFrontBuffer == nullptr;
   }
 
-  void ReadUnlock()
-  {
-    MOZ_ASSERT(mFrontLock, "ReadLock with no gfxSharedReadLock");
-    if (mFrontLock) {
-      mFrontLock->ReadUnlock();
-    }
-  }
-
-  void ReadLock()
-  {
-    MOZ_ASSERT(mFrontLock, "ReadLock with no gfxSharedReadLock");
-    if (mFrontLock) {
-      mFrontLock->ReadLock();
-    }
-  }
-
   void DiscardBuffers()
   {
     DiscardFrontBuffer();
@@ -244,7 +117,8 @@ struct TileClient
   *
   * If nullptr is returned, aTextureClientOnWhite is undefined.
   */
-  TextureClient* GetBackBuffer(const nsIntRegion& aDirtyRegion,
+  TextureClient* GetBackBuffer(CompositableClient&,
+                               const nsIntRegion& aDirtyRegion,
                                gfxContentType aContent, SurfaceMode aMode,
                                nsIntRegion& aAddPaintedRegion,
                                RefPtr<TextureClient>* aTextureClientOnWhite);
@@ -271,12 +145,9 @@ struct TileClient
   RefPtr<TextureClient> mBackBufferOnWhite;
   RefPtr<TextureClient> mFrontBuffer;
   RefPtr<TextureClient> mFrontBufferOnWhite;
-  RefPtr<gfxSharedReadLock> mBackLock;
-  RefPtr<gfxSharedReadLock> mFrontLock;
-  RefPtr<ClientLayerManager> mManager;
   RefPtr<TextureClientAllocator> mAllocator;
   gfx::IntRect mUpdateRect;
-  CompositableClient* mCompositableClient;
+  bool mWasPlaceholder;
 #ifdef GFX_TILEDLAYER_DEBUG_OVERLAY
   TimeStamp        mLastUpdate;
 #endif
@@ -318,10 +189,10 @@ struct BasicTiledLayerPaintData {
 
   /*
    * The critical displayport of the content from the nearest ancestor layer
-   * that represents scrollable content with a display port set. Empty if a
-   * critical displayport is not set.
+   * that represents scrollable content with a display port set. isNothing()
+   * if a critical displayport is not set.
    */
-  LayerIntRect mCriticalDisplayPort;
+  Maybe<LayerIntRect> mCriticalDisplayPort;
 
   /*
    * The render resolution of the document that the content this layer
@@ -355,6 +226,11 @@ struct BasicTiledLayerPaintData {
    */
   bool mPaintFinished : 1;
 
+  /*
+   * Whether or not there is an async transform animation active
+   */
+  bool mHasTransformAnimation : 1;
+
   /*
    * Initializes/clears data to prepare for paint action.
    */
@@ -377,7 +253,7 @@ public:
   bool UpdateFromCompositorFrameMetrics(const LayerMetricsWrapper& aLayer,
                                         bool aHasPendingNewThebesContent,
                                         bool aLowPrecision,
-                                        ViewTransform& aViewTransform);
+                                        AsyncTransform& aViewTransform);
 
   /**
    * Determines if the compositor's upcoming composition bounds has fallen
@@ -403,19 +279,21 @@ private:
 class ClientTiledLayerBuffer
 {
 public:
-  ClientTiledLayerBuffer(ClientTiledPaintedLayer* aPaintedLayer,
-                         CompositableClient* aCompositableClient)
+  ClientTiledLayerBuffer(ClientTiledPaintedLayer& aPaintedLayer,
+                         CompositableClient& aCompositableClient)
     : mPaintedLayer(aPaintedLayer)
     , mCompositableClient(aCompositableClient)
     , mLastPaintContentType(gfxContentType::COLOR)
     , mLastPaintSurfaceMode(SurfaceMode::SURFACE_OPAQUE)
+    , mWasLastPaintProgressive(false)
   {}
 
   virtual void PaintThebes(const nsIntRegion& aNewValidRegion,
                    const nsIntRegion& aPaintRegion,
                    const nsIntRegion& aDirtyRegion,
                    LayerManager::DrawPaintedLayerCallback aCallback,
-                   void* aCallbackData) = 0;
+                   void* aCallbackData,
+                   bool aIsProgressive = false) = 0;
 
   virtual bool SupportsProgressiveUpdate() = 0;
   virtual bool ProgressiveUpdate(nsIntRegion& aValidRegion,
@@ -444,12 +322,14 @@ protected:
   void UnlockTile(TileClient& aTile);
   gfxContentType GetContentType(SurfaceMode* aMode = nullptr) const;
 
-  ClientTiledPaintedLayer* mPaintedLayer;
-  CompositableClient* mCompositableClient;
+  ClientTiledPaintedLayer& mPaintedLayer;
+  CompositableClient& mCompositableClient;
 
   gfxContentType mLastPaintContentType;
   SurfaceMode mLastPaintSurfaceMode;
   CSSToParentLayerScale2D mFrameResolution;
+
+  bool mWasLastPaintProgressive;
 };
 
 class ClientMultiTiledLayerBuffer
@@ -458,25 +338,17 @@ class ClientMultiTiledLayerBuffer
 {
   friend class TiledLayerBuffer<ClientMultiTiledLayerBuffer, TileClient>;
 public:
-  ClientMultiTiledLayerBuffer(ClientTiledPaintedLayer* aPaintedLayer,
-                              CompositableClient* aCompositableClient,
+  ClientMultiTiledLayerBuffer(ClientTiledPaintedLayer& aPaintedLayer,
+                              CompositableClient& aCompositableClient,
                               ClientLayerManager* aManager,
                               SharedFrameMetricsHelper* aHelper);
-  ClientMultiTiledLayerBuffer()
-    : ClientTiledLayerBuffer(nullptr, nullptr)
-    , mManager(nullptr)
-    , mCallback(nullptr)
-    , mCallbackData(nullptr)
-    , mSharedFrameMetricsHelper(nullptr)
-    , mTilingOrigin(std::numeric_limits<int32_t>::max(),
-                    std::numeric_limits<int32_t>::max())
-  {}
 
   void PaintThebes(const nsIntRegion& aNewValidRegion,
                    const nsIntRegion& aPaintRegion,
                    const nsIntRegion& aDirtyRegion,
                    LayerManager::DrawPaintedLayerCallback aCallback,
-                   void* aCallbackData) override;
+                   void* aCallbackData,
+                   bool aIsProgressive = false) override;
 
   virtual bool SupportsProgressiveUpdate() override { return true; }
   /**
@@ -544,7 +416,7 @@ protected:
   TileClient GetPlaceholderTile() const { return TileClient(); }
 
 private:
-  ClientLayerManager* mManager;
+  RefPtr<ClientLayerManager> mManager;
   LayerManager::DrawPaintedLayerCallback mCallback;
   void* mCallbackData;
 
@@ -552,9 +424,6 @@ private:
   // completed then this is identical to mValidRegion.
   nsIntRegion mNewValidRegion;
 
-  // The DrawTarget we use when UseSinglePaintBuffer() above is true.
-  RefPtr<gfx::DrawTarget>       mSinglePaintDrawTarget;
-  nsIntPoint                    mSinglePaintBufferOffset;
   SharedFrameMetricsHelper*  mSharedFrameMetricsHelper;
   // When using Moz2D's CreateTiledDrawTarget we maintain a list of gfx::Tiles
   std::vector<gfx::Tile> mMoz2DTiles;
@@ -628,9 +497,6 @@ public:
   };
   virtual void UpdatedBuffer(TiledBufferType aType) = 0;
 
-  virtual bool SupportsLayerSize(const gfx::IntSize& aSize, ClientLayerManager* aManager) const
-  { return true; }
-
 private:
   const char* mName;
 };
@@ -642,7 +508,7 @@ private:
 class MultiTiledContentClient : public TiledContentClient
 {
 public:
-  MultiTiledContentClient(ClientTiledPaintedLayer* aPaintedLayer,
+  MultiTiledContentClient(ClientTiledPaintedLayer& aPaintedLayer,
                           ClientLayerManager* aManager);
 
 protected:
