@@ -11,10 +11,16 @@
 #include "mozilla/mozalloc.h"
 #include "mozilla/mozalloc_abort.h"
 #include "mozilla/mozalloc_oom.h"
+#include "prenv.h"
 
-void sk_throw() {
-    SkDEBUGFAIL("sk_throw");
-    mozalloc_abort("Abort from sk_throw");
+void sk_abort_no_print() {
+#ifdef SK_DEBUG
+    const char* env = PR_GetEnv("MOZ_SKIA_DISABLE_ASSERTS");
+    if (env && *env != '0') {
+        return;
+    }
+#endif
+    mozalloc_abort("Abort from sk_abort");
 }
 
 void sk_out_of_memory(void) {
