@@ -4,8 +4,8 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 463334;
+var summary = 'TM: Do not crash in isPromoteInt';
 var actual = '';
 var expect = '';
 
@@ -19,23 +19,11 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
+ 
 
-// ------- Comment #130 From Gary Kwong [:nth10sd]
-
-// Does not require -j:
-// =====
-  ((function x()x in []) for (y in []))
-
-// Assertion failure: !(pnu->pn_dflags & PND_BOUND), at ../jsemit.cpp:1818
-// =====
-
-
-// Requires -j:
-// =====
-  for (var x = 0; x < 3; ++x) { new function(){} }
-
-// Assertion failure: cx->bailExit, at ../jstracer.cpp:4672
-// Opt crash [@ LeaveTree] near null
+  u = 3;
+  for (let i in (function() { for (var j=0;j<4;++j) { void u; yield; } })());
+  
 
   reportCompare(expect, actual, summary);
 
