@@ -59,6 +59,64 @@ var mediaPlayerDevice = {
   }
 };
 
+var fxOSTVDevice = {
+  id: "app://fling-player.gaiamobile.org",
+  target: "app://fling-player.gaiamobile.org/index.html",
+  factory: function(aService) {
+    Cu.import("resource://gre/modules/PresentationApp.jsm");
+    let request = new window.PresentationRequest(this.target);
+    return new PresentationApp(aService, request);
+  },
+  init: function() {
+    Services.obs.addObserver(this, "presentation-device-change", false);
+    SimpleServiceDiscovery.addExternalDiscovery(this);
+  },
+  observe: function(subject, topic, data) {
+    let device = subject.QueryInterface(Ci.nsIPresentationDevice);
+    let service = this.toService(device);
+    switch (data) {
+      case "add":
+        SimpleServiceDiscovery.addService(service);
+        break;
+      case "update":
+        SimpleServiceDiscovery.updateService(service);
+        break;
+      case "remove":
+        if(SimpleServiceDiscovery.findServiceForID(device.id)) {
+          SimpleServiceDiscovery.removeService(device.id);
+        }
+        break;
+    }
+  },
+  toService: function(device) {
+    return {
+      location: device.id,
+      target: fxOSTVDevice.target,
+      friendlyName: device.name,
+      uuid: device.id,
+      manufacturer: "Firefox OS TV",
+      modelName: "Firefox OS TV",
+    };
+  },
+  startDiscovery: function() {
+    window.navigator.mozPresentationDeviceInfo.forceDiscovery();
+
+    // need to update the lastPing time for known device.
+    window.navigator.mozPresentationDeviceInfo.getAll()
+    .then(function(devices) {
+      for (let device of devices) {
+        let service = fxOSTVDevice.toService(device);
+        SimpleServiceDiscovery.addService(service);
+      }
+    });
+  },
+  stopDiscovery: function() {
+    // do nothing
+  },
+  types: ["video/mp4", "video/webm"],
+  extensions: ["mp4", "webm"],
+};
+
 var CastingApps = {
   _castMenuId: -1,
   mirrorStartMenuId: -1,
@@ -79,6 +137,12 @@ var CastingApps = {
     mediaPlayerDevice.init();
     SimpleServiceDiscovery.registerDevice(mediaPlayerDevice);
 
+    // Presentation Device will notify us any time the available device list changes.
+    if (window.PresentationRequest) {
+      fxOSTVDevice.init();
+      SimpleServiceDiscovery.registerDevice(fxOSTVDevice);
+    }
+
     // Search for devices continuously
     SimpleServiceDiscovery.search(this._interval);
 
@@ -352,16 +416,23 @@ var CastingApps = {
   _getContentTypeForURI: function(aURI, aElement, aCallback) {
     let channel;
     try {
-     channel = Services.io.newChannelFromURI2(aURI,
-                                              aElement,
-                                              null, // aLoadingPrincipal
-                                              null, // aTriggeringPrincipal
-                                              Ci.nsILoadInfo.SEC_NORMAL,
-                                              Ci.nsIContentPolicy.TYPE_OTHER);
-     } catch(e) {
-      aCallback(null);
-      return;
-     }
+      let secFlags = Ci.nsILoadInfo.SEC_ALLOW_CROSS_ORIGIN_DATA_INHERITS;
+      if (aElement.crossOrigin) {
+        secFlags = Ci.nsILoadInfo.SEC_REQUIRE_CORS_DATA_INHERITS;
+        if (aElement.crossOrigin === "use-credentials") {
+          secFlags |= Ci.nsILoadInfo.SEC_COOKIES_INCLUDE;
+        }
+      }
+      channel = NetUtil.newChannel({
+        uri: aURI,
+        loadingNode: aElement,
+        securityFlags: secFlags,
+        contentPolicyType: Ci.nsIContentPolicy.TYPE_INTERNAL_VIDEO
+      });
+    } catch(e) {
+     aCallback(null);
+     return;
+    }
 
     let listener = {
       onStartRequest: function(request, context) {
@@ -384,7 +455,7 @@ var CastingApps = {
     };
 
     if (channel) {
-      channel.asyncOpen(listener, null);
+      channel.asyncOpen2(listener);
     } else {
       aCallback(null);
     }
