@@ -137,6 +137,8 @@ public class testDistribution extends ContentProviderTest {
 
         // Pre-clear distribution pref, run basic preferences and en-US localized preferences Tests
         clearDistributionPref();
+        clearDistributionFromDataData();
+
         setTestLocale("en-US");
         try {
             initDistribution(mockPackagePath);
@@ -154,6 +156,7 @@ public class testDistribution extends ContentProviderTest {
 
         // Pre-clear distribution pref, and run es-MX localized preferences Test
         clearDistributionPref();
+        clearDistributionFromDataData();
         setTestLocale("es-MX");
         initDistribution(mockPackagePath);
         checkLocalizedPreferences("es-MX");
@@ -161,9 +164,11 @@ public class testDistribution extends ContentProviderTest {
         // Test the (stubbed) download interaction.
         setTestLocale("en-US");
         clearDistributionPref();
+        clearDistributionFromDataData();
         doTestValidReferrerIntent();
 
         clearDistributionPref();
+        clearDistributionFromDataData();
         doTestInvalidReferrerIntent();
     }
 
@@ -280,9 +285,9 @@ public class testDistribution extends ContentProviderTest {
         // Call Distribution.init with the mock package.
         Actions.EventExpecter distributionSetExpecter = mActions.expectGeckoEvent("Distribution:Set:OK");
         Distribution dist = Distribution.init(mActivity, aPackagePath, "prefs-" + System.currentTimeMillis());
-        DistroSharedPrefsImport.importPreferences(mActivity, dist);
         distributionSetExpecter.blockForEvent();
         distributionSetExpecter.unregisterListener();
+        DistroSharedPrefsImport.importPreferences(mActivity, dist);
         return dist;
     }
 
@@ -392,20 +397,36 @@ public class testDistribution extends ContentProviderTest {
     }
 
     private JSONArray getPrefs(String[] prefNames) throws JSONException {
-        Actions.RepeatedEventExpecter eventExpecter = mActions.expectGeckoEvent("Preferences:Data");
-        mActions.sendPreferencesGetEvent(PREF_REQUEST_ID, prefNames);
+        final JSONArray result = new JSONArray();
 
-        JSONObject data = null;
-        int requestId = -1;
+        mActions.getPrefs(prefNames, new Actions.PrefHandlerBase() {
+            private void addItem(String pref, Object value) {
+                try {
+                    final JSONObject item = new JSONObject();
+                    item.put("name", pref).put("value", value);
+                    result.put(item);
+                } catch (final JSONException e) {
+                    mAsserter.ok(false, "exception getting prefs", e.toString());
+                }
+            }
 
-        // Wait until we get the correct "Preferences:Data" event
-        while (requestId != PREF_REQUEST_ID) {
-            data = new JSONObject(eventExpecter.blockForEventData());
-            requestId = data.getInt("requestId");
-        }
-        eventExpecter.unregisterListener();
+            @Override // Actions.PrefHandlerBase
+            public void prefValue(String pref, boolean value) {
+                addItem(pref, value);
+            }
 
-        return data.getJSONArray("preferences");
+            @Override // Actions.PrefHandlerBase
+            public void prefValue(String pref, int value) {
+                addItem(pref, value);
+            }
+
+            @Override // Actions.PrefHandlerBase
+            public void prefValue(String pref, String value) {
+                addItem(pref, value);
+            }
+        }).waitForFinish();
+
+        return result;
     }
 
     // Sets the distribution locale preference for the test.
@@ -414,56 +435,39 @@ public class testDistribution extends ContentProviderTest {
     }
 
     // Test localized distribution and preferences values stored in preferences.json
-    private void checkLocalizedPreferences(String aLocale) {
-        String prefAbout = "distribution.about";
-        String prefLocalizeable = "distribution.test.localizeable";
-        String prefLocalizeableOverride = "distribution.test.localizeable-override";
-
-        try {
-            final String[] prefNames = { prefAbout, prefLocalizeable, prefLocalizeableOverride };
-
-            Actions.RepeatedEventExpecter eventExpecter = mActions.expectGeckoEvent("Preferences:Data");
-            mActions.sendPreferencesGetEvent(PREF_REQUEST_ID, prefNames);
-
-            JSONObject data = null;
-            int requestId = -1;
-
-            // Wait until we get the correct "Preferences:Data" event
-            while (requestId != PREF_REQUEST_ID) {
-                data = new JSONObject(eventExpecter.blockForEventData());
-                requestId = data.getInt("requestId");
-            }
-            eventExpecter.unregisterListener();
-
-            JSONArray preferences = data.getJSONArray("preferences");
-            for (int i = 0; i < preferences.length(); i++) {
-                JSONObject pref = (JSONObject) preferences.get(i);
-                String name = pref.getString("name");
+    private void checkLocalizedPreferences(final String aLocale) {
+        final String prefAbout = "distribution.about";
+        final String prefLocalizeable = "distribution.test.localizeable";
+        final String prefLocalizeableOverride = "distribution.test.localizeable-override";
+        final String[] prefNames = { prefAbout, prefLocalizeable, prefLocalizeableOverride };
 
+        mActions.getPrefs(prefNames, new Actions.PrefHandlerBase() {
+            @Override // Actions.PrefHandlerBase
+            public void prefValue(String name, String value) {
                 if (name.equals(prefAbout)) {
                     if (aLocale.equals("en-US")) {
-                        mAsserter.is(pref.getString("value"), "Test Partner", "check " + prefAbout);
+                        mAsserter.is(value, "Test Partner", "check " + prefAbout);
                     } else if (aLocale.equals("es-MX")) {
-                        mAsserter.is(pref.getString("value"), "Afiliado de Prueba", "check " + prefAbout);
+                        mAsserter.is(value, "Afiliado de Prueba", "check " + prefAbout);
                     }
                 } else if (name.equals(prefLocalizeable)) {
                     if (aLocale.equals("en-US")) {
-                        mAsserter.is(pref.getString("value"), "http://test.org/en-US/en-US/", "check " + prefLocalizeable);
+                        mAsserter.is(value, "http://test.org/en-US/en-US/", "check " + prefLocalizeable);
                     } else if (aLocale.equals("es-MX")) {
-                        mAsserter.is(pref.getString("value"), "http://test.org/es-MX/es-MX/", "check " + prefLocalizeable);
+                        mAsserter.is(value, "http://test.org/es-MX/es-MX/", "check " + prefLocalizeable);
                     }
                 } else if (name.equals(prefLocalizeableOverride)) {
                     if (aLocale.equals("en-US")) {
-                        mAsserter.is(pref.getString("value"), "http://cheese.com", "check " + prefLocalizeableOverride);
+                        mAsserter.is(value, "http://cheese.com", "check " + prefLocalizeableOverride);
                     } else if (aLocale.equals("es-MX")) {
-                        mAsserter.is(pref.getString("value"), "http://test.org/es-MX/", "check " + prefLocalizeableOverride);
+                        mAsserter.is(value, "http://test.org/es-MX/", "check " + prefLocalizeableOverride);
                     }
+                } else {
+                    // Raise exception.
+                    super.prefValue(name, value);
                 }
             }
-
-        } catch (JSONException e) {
-            mAsserter.ok(false, "exception getting preferences", e.toString());
-        }
+        }).waitForFinish();
     }
 
     // Copies the mock package to the data directory and returns the file path to it.
@@ -504,6 +508,22 @@ public class testDistribution extends ContentProviderTest {
         TestableDistribution.clearReferrerDescriptorForTesting();
     }
 
+    /**
+     * Clears any distribution found in /data/data.
+     */
+    private void clearDistributionFromDataData() throws Exception {
+        File dataDir = new File(mActivity.getApplicationInfo().dataDir);
+
+        // Recursively delete distribution files that Distribution.init copied to data directory.
+        File distDir = new File(dataDir, "distribution");
+        if (distDir.exists()) {
+            mAsserter.dumpLog("Clearing distribution from " + distDir.getAbsolutePath());
+            delete(distDir);
+        } else {
+            mAsserter.dumpLog("No distribution to clear from " + distDir.getAbsolutePath());
+        }
+    }
+
     @Override
     public void setUp() throws Exception {
         // TODO: Set up the content provider after setting the distribution.
@@ -528,10 +548,7 @@ public class testDistribution extends ContentProviderTest {
         File mockPackage = new File(dataDir, MOCK_PACKAGE);
         mAsserter.ok(mockPackage.delete(), "clean up mock package", "deleted " + mockPackage.getPath());
 
-        // Recursively delete distribution files that Distribution.init copied to data directory.
-        File distDir = new File(dataDir, "distribution");
-        delete(distDir);
-
+        clearDistributionFromDataData();
         clearDistributionPref();
 
         super.tearDown();
