@@ -10,6 +10,7 @@
 #include "mozilla/ArrayUtils.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/HashFunctions.h"
+#include "mozilla/UniquePtrExtensions.h"
 
 #include "nsXULAppAPI.h"
 
@@ -74,6 +75,14 @@ namespace mozilla {
 #define INITIAL_PREF_FILES 10
 static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 
+void
+Preferences::DirtyCallback()
+{
+  if (gHashTable && sPreferences && !sPreferences->mDirty) {
+    sPreferences->mDirty = true;
+  }
+}
+
 // Prototypes
 static nsresult openPrefFile(nsIFile* aFile);
 static nsresult pref_InitInitialObjects(void);
@@ -84,6 +93,26 @@ static const char kTelemetryPref[] = "toolkit.telemetry.enabled";
 static const char kOldTelemetryPref[] = "toolkit.telemetry.enabledPreRelease";
 static const char kChannelPref[] = "app.update.channel";
 
+static const char kPrefFileHeader[] =
+  "# Mozilla User Preferences"
+  NS_LINEBREAK
+  NS_LINEBREAK
+  "/* Do not edit this file."
+  NS_LINEBREAK
+  " *"
+  NS_LINEBREAK
+  " * If you make changes to this file while the application is running,"
+  NS_LINEBREAK
+  " * the changes will be overwritten when the application exits."
+  NS_LINEBREAK
+  " *"
+  NS_LINEBREAK
+  " * To make a manual change to preferences, you can visit the URL about:config"
+  NS_LINEBREAK
+  " */"
+  NS_LINEBREAK
+  NS_LINEBREAK;
+
 Preferences* Preferences::sPreferences = nullptr;
 nsIPrefBranch* Preferences::sRootBranch = nullptr;
 nsIPrefBranch* Preferences::sDefaultRootBranch = nullptr;
@@ -102,19 +131,24 @@ public:
   static PLDHashNumber HashKey(const ValueObserverHashKey *aKey)
   {
     PLDHashNumber hash = HashString(aKey->mPrefName);
+    hash = AddToHash(hash, aKey->mMatchKind);
     return AddToHash(hash, aKey->mCallback);
   }
 
-  ValueObserverHashKey(const char *aPref, PrefChangedFunc aCallback) :
-    mPrefName(aPref), mCallback(aCallback) { }
+  ValueObserverHashKey(const char *aPref, PrefChangedFunc aCallback, Preferences::MatchKind aMatchKind) :
+    mPrefName(aPref), mCallback(aCallback), mMatchKind(aMatchKind) { }
 
   explicit ValueObserverHashKey(const ValueObserverHashKey *aOther) :
-    mPrefName(aOther->mPrefName), mCallback(aOther->mCallback)
+    mPrefName(aOther->mPrefName),
+    mCallback(aOther->mCallback),
+    mMatchKind(aOther->mMatchKind)
   { }
 
   bool KeyEquals(const ValueObserverHashKey *aOther) const
   {
-    return mCallback == aOther->mCallback && mPrefName == aOther->mPrefName;
+    return mCallback == aOther->mCallback &&
+           mPrefName == aOther->mPrefName &&
+           mMatchKind == aOther->mMatchKind;
   }
 
   ValueObserverHashKey *GetKey() const
@@ -126,6 +160,7 @@ public:
 
   nsCString mPrefName;
   PrefChangedFunc mCallback;
+  Preferences::MatchKind mMatchKind;
 };
 
 class ValueObserver final : public nsIObserver,
@@ -139,8 +174,8 @@ public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIOBSERVER
 
-  ValueObserver(const char *aPref, PrefChangedFunc aCallback)
-    : ValueObserverHashKey(aPref, aCallback) { }
+  ValueObserver(const char *aPref, PrefChangedFunc aCallback, Preferences::MatchKind aMatchKind)
+    : ValueObserverHashKey(aPref, aCallback, aMatchKind) { }
 
   void AppendClosure(void *aClosure) {
     mClosures.AppendElement(aClosure);
@@ -167,6 +202,9 @@ ValueObserver::Observe(nsISupports     *aSubject,
   NS_ASSERTION(!nsCRT::strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID),
                "invalid topic");
   NS_ConvertUTF16toUTF8 data(aData);
+  if (mMatchKind == Preferences::ExactMatch && !mPrefName.EqualsASCII(data.get())) {
+    return NS_OK;
+  }
   for (uint32_t i = 0; i < mClosures.Length(); i++) {
     mCallback(data.get(), mClosures.ElementAt(i));
   }
@@ -218,6 +256,14 @@ AssertNotAlreadyCached(const char* aPrefType,
 }
 #endif
 
+static void
+ReportToConsole(const char* aMessage, int aLine, bool aError)
+{
+  nsPrintfCString message("** Preference parsing %s (line %d) = %s **\n",
+                          (aError ? "error" : "warning"), aLine, aMessage);
+  nsPrefBranch::ReportToConsole(NS_ConvertUTF8toUTF16(message.get()));
+}
+
 // Although this is a member of Preferences, it measures sPreferences and
 // several other global structures.
 /* static */ int64_t
@@ -238,15 +284,18 @@ Preferences::SizeOfIncludingThisAndOtherStuff(mozilla::MallocSizeOf aMallocSizeO
     }
   }
   if (gObserverTable) {
-    n += aMallocSizeOf(gObserverTable);
     n += gObserverTable->ShallowSizeOfIncludingThis(aMallocSizeOf);
     for (auto iter = gObserverTable->Iter(); !iter.Done(); iter.Next()) {
       n += iter.Key()->mPrefName.SizeOfExcludingThisIfUnshared(aMallocSizeOf);
       n += iter.Data()->mClosures.ShallowSizeOfExcludingThis(aMallocSizeOf);
     }
   }
-  // We don't measure sRootBranch and sDefaultRootBranch here because
-  // DMD indicates they are not significant.
+  if (sRootBranch) {
+    n += reinterpret_cast<nsPrefBranch*>(sRootBranch)->SizeOfIncludingThis(aMallocSizeOf);
+  }
+  if (sDefaultRootBranch) {
+    n += reinterpret_cast<nsPrefBranch*>(sDefaultRootBranch)->SizeOfIncludingThis(aMallocSizeOf);
+  }
   n += pref_SizeOfPrivateData(aMallocSizeOf);
   return n;
 }
@@ -268,23 +317,13 @@ NS_IMPL_ISUPPORTS(PreferenceServiceReporter, nsIMemoryReporter)
 MOZ_DEFINE_MALLOC_SIZE_OF(PreferenceServiceMallocSizeOf)
 
 NS_IMETHODIMP
-PreferenceServiceReporter::CollectReports(nsIMemoryReporterCallback* aCb,
-                                          nsISupports* aClosure,
-                                          bool aAnonymize)
+PreferenceServiceReporter::CollectReports(
+  nsIHandleReportCallback* aHandleReport, nsISupports* aData, bool aAnonymize)
 {
-#define REPORT(_path, _kind, _units, _amount, _desc)                          \
-    do {                                                                      \
-      nsresult rv;                                                            \
-      rv = aCb->Callback(EmptyCString(), _path, _kind,                        \
-                         _units, _amount, NS_LITERAL_CSTRING(_desc),          \
-                         aClosure);                                           \
-      NS_ENSURE_SUCCESS(rv, rv);                                              \
-    } while (0)
-
-  REPORT(NS_LITERAL_CSTRING("explicit/preferences"),
-         KIND_HEAP, UNITS_BYTES,
-         Preferences::SizeOfIncludingThisAndOtherStuff(PreferenceServiceMallocSizeOf),
-         "Memory used by the preferences system.");
+  MOZ_COLLECT_REPORT(
+    "explicit/preferences", KIND_HEAP, UNITS_BYTES,
+    Preferences::SizeOfIncludingThisAndOtherStuff(PreferenceServiceMallocSizeOf),
+    "Memory used by the preferences system.");
 
   nsPrefBranch* rootBranch =
     static_cast<nsPrefBranch*>(Preferences::GetRootBranch());
@@ -336,35 +375,39 @@ PreferenceServiceReporter::CollectReports(nsIMemoryReporterCallback* aCb,
     nsPrintfCString suspectPath("preference-service-suspect/"
                                 "referent(pref=%s)", suspect.get());
 
-    REPORT(suspectPath,
-           KIND_OTHER, UNITS_COUNT, totalReferentCount,
-           "A preference with a suspiciously large number "
-           "referents (symptom of a leak).");
+    aHandleReport->Callback(
+      /* process = */ EmptyCString(),
+      suspectPath, KIND_OTHER, UNITS_COUNT, totalReferentCount,
+      NS_LITERAL_CSTRING(
+        "A preference with a suspiciously large number referents (symptom of a "
+        "leak)."),
+      aData);
   }
 
-  REPORT(NS_LITERAL_CSTRING("preference-service/referent/strong"),
-         KIND_OTHER, UNITS_COUNT, numStrong,
-         "The number of strong referents held by the preference service.");
+  MOZ_COLLECT_REPORT(
+    "preference-service/referent/strong", KIND_OTHER, UNITS_COUNT,
+    numStrong,
+    "The number of strong referents held by the preference service.");
 
-  REPORT(NS_LITERAL_CSTRING("preference-service/referent/weak/alive"),
-         KIND_OTHER, UNITS_COUNT, numWeakAlive,
-         "The number of weak referents held by the preference service "
-         "that are still alive.");
+  MOZ_COLLECT_REPORT(
+    "preference-service/referent/weak/alive", KIND_OTHER, UNITS_COUNT,
+    numWeakAlive,
+    "The number of weak referents held by the preference service that are "
+    "still alive.");
 
-  REPORT(NS_LITERAL_CSTRING("preference-service/referent/weak/dead"),
-         KIND_OTHER, UNITS_COUNT, numWeakDead,
-         "The number of weak referents held by the preference service "
-         "that are dead.");
-
-#undef REPORT
+  MOZ_COLLECT_REPORT(
+    "preference-service/referent/weak/dead", KIND_OTHER, UNITS_COUNT,
+    numWeakDead,
+    "The number of weak referents held by the preference service that are "
+    "dead.");
 
   return NS_OK;
 }
 
 namespace {
-class AddPreferencesMemoryReporterRunnable : public nsRunnable
+class AddPreferencesMemoryReporterRunnable : public Runnable
 {
-  NS_IMETHOD Run()
+  NS_IMETHOD Run() override
   {
     return RegisterStrongMemoryReporter(new PreferenceServiceReporter());
   }
@@ -412,6 +455,13 @@ Preferences::GetInstanceForService()
   return sPreferences;
 }
 
+// static
+bool
+Preferences::IsServiceAvailable()
+{
+  return !!sPreferences;
+}
+
 // static
 bool
 Preferences::InitStaticMembers()
@@ -451,6 +501,7 @@ Preferences::Shutdown()
  */
 
 Preferences::Preferences()
+  : mDirty(false)
 {
 }
 
@@ -500,6 +551,7 @@ Preferences::Init()
 {
   nsresult rv;
 
+  PREF_SetDirtyCallback(&DirtyCallback);
   PREF_Init();
 
   rv = pref_InitInitialObjects();
@@ -679,7 +731,7 @@ ReadExtensionPrefs(nsIFile *aFile)
     uint32_t read;
 
     PrefParseState ps;
-    PREF_InitParseState(&ps, PREF_ReaderCallback, nullptr);
+    PREF_InitParseState(&ps, PREF_ReaderCallback, ReportToConsole, nullptr);
     while (NS_SUCCEEDED(rv = stream->Available(&avail)) && avail) {
       rv = stream->Read(buffer, 4096, &read);
       if (NS_FAILED(rv)) {
@@ -707,7 +759,9 @@ Preferences::GetPreference(PrefSetting* aPref)
   if (!entry)
     return;
 
-  pref_GetPrefFromEntry(entry, aPref);
+  if (pref_EntryHasAdvisablySizedValues(entry)) {
+    pref_GetPrefFromEntry(entry, aPref);
+  }
 }
 
 void
@@ -716,6 +770,11 @@ Preferences::GetPreferences(InfallibleTArray<PrefSetting>* aPrefs)
   aPrefs->SetCapacity(gHashTable->Capacity());
   for (auto iter = gHashTable->Iter(); !iter.Done(); iter.Next()) {
     auto entry = static_cast<PrefHashEntry*>(iter.Get());
+
+    if (!pref_EntryHasAdvisablySizedValues(entry)) {
+      continue;
+    }
+
     dom::PrefSetting *pref = aPrefs->AppendElement();
     pref_GetPrefFromEntry(entry, pref);
   }
@@ -760,7 +819,7 @@ Preferences::GetDefaultBranch(const char *aPrefRoot, nsIPrefBranch **_retval)
 
 NS_IMETHODIMP
 Preferences::GetDirty(bool *_retval) {
-  *_retval = gDirty;
+  *_retval = mDirty;
   return NS_OK;
 }
 
@@ -879,11 +938,12 @@ nsresult
 Preferences::SavePrefFileInternal(nsIFile *aFile)
 {
   if (nullptr == aFile) {
-    // the gDirty flag tells us if we should write to mCurrentFile
+    // the mDirty flag tells us if we should write to mCurrentFile
     // we only check this flag when the caller wants to write to the default
-    if (!gDirty)
+    if (!mDirty) {
       return NS_OK;
-    
+    }
+
     // It's possible that we never got a prefs file.
     nsresult rv = NS_OK;
     if (mCurrentFile)
@@ -898,26 +958,6 @@ Preferences::SavePrefFileInternal(nsIFile *aFile)
 nsresult
 Preferences::WritePrefFile(nsIFile* aFile)
 {
-  const char                outHeader[] =
-    "# Mozilla User Preferences"
-    NS_LINEBREAK
-    NS_LINEBREAK
-    "/* Do not edit this file."
-    NS_LINEBREAK
-    " *"
-    NS_LINEBREAK
-    " * If you make changes to this file while the application is running,"
-    NS_LINEBREAK
-    " * the changes will be overwritten when the application exits."
-    NS_LINEBREAK
-    " *"
-    NS_LINEBREAK
-    " * To make a manual change to preferences, you can visit the URL about:config"
-    NS_LINEBREAK
-    " */"
-    NS_LINEBREAK
-    NS_LINEBREAK;
-
   nsCOMPtr<nsIOutputStream> outStreamSink;
   nsCOMPtr<nsIOutputStream> outStream;
   uint32_t                  writeAmount;
@@ -937,26 +977,24 @@ Preferences::WritePrefFile(nsIFile* aFile)
   if (NS_FAILED(rv)) 
       return rv;  
 
-  nsAutoArrayPtr<char*> valueArray(new char*[gHashTable->EntryCount()]);
-  memset(valueArray, 0, gHashTable->EntryCount() * sizeof(char*));
-
   // get the lines that we're supposed to be writing to the file
-  pref_savePrefs(gHashTable, valueArray);
+  uint32_t prefCount;
+  UniquePtr<char*[]> valueArray = pref_savePrefs(gHashTable, &prefCount);
 
   /* Sort the preferences to make a readable file on disk */
-  NS_QuickSort(valueArray, gHashTable->EntryCount(), sizeof(char *),
+  NS_QuickSort(valueArray.get(), prefCount, sizeof(char *),
                pref_CompareStrings, nullptr);
 
   // write out the file header
-  outStream->Write(outHeader, sizeof(outHeader) - 1, &writeAmount);
+  outStream->Write(kPrefFileHeader, sizeof(kPrefFileHeader) - 1, &writeAmount);
 
-  char** walker = valueArray;
-  for (uint32_t valueIdx = 0; valueIdx < gHashTable->EntryCount(); valueIdx++, walker++) {
-    if (*walker) {
-      outStream->Write(*walker, strlen(*walker), &writeAmount);
-      outStream->Write(NS_LINEBREAK, NS_LINEBREAK_LEN, &writeAmount);
-      free(*walker);
-    }
+  for (uint32_t valueIdx = 0; valueIdx < prefCount; valueIdx++) {
+    char*& pref = valueArray[valueIdx];
+    MOZ_ASSERT(pref);
+    outStream->Write(pref, strlen(pref), &writeAmount);
+    outStream->Write(NS_LINEBREAK, NS_LINEBREAK_LEN, &writeAmount);
+    free(pref);
+    pref = nullptr;
   }
 
   // tell the safe output stream to overwrite the real prefs file
@@ -971,7 +1009,7 @@ Preferences::WritePrefFile(nsIFile* aFile)
     }
   }
 
-  gDirty = false;
+  mDirty = false;
   return NS_OK;
 }
 
@@ -990,12 +1028,12 @@ static nsresult openPrefFile(nsIFile* aFile)
   NS_ENSURE_TRUE(fileSize64 <= UINT32_MAX, NS_ERROR_FILE_TOO_BIG);
 
   uint32_t fileSize = (uint32_t)fileSize64;
-  nsAutoArrayPtr<char> fileBuffer(new char[fileSize]);
+  auto fileBuffer = MakeUniqueFallible<char[]>(fileSize);
   if (fileBuffer == nullptr)
     return NS_ERROR_OUT_OF_MEMORY;
 
   PrefParseState ps;
-  PREF_InitParseState(&ps, PREF_ReaderCallback, nullptr);
+  PREF_InitParseState(&ps, PREF_ReaderCallback, ReportToConsole, nullptr);
 
   // Read is not guaranteed to return a buf the size of fileSize,
   // but usually will.
@@ -1003,10 +1041,10 @@ static nsresult openPrefFile(nsIFile* aFile)
   uint32_t offset = 0;
   for (;;) {
     uint32_t amtRead = 0;
-    rv = inStr->Read((char*)fileBuffer, fileSize, &amtRead);
+    rv = inStr->Read(fileBuffer.get(), fileSize, &amtRead);
     if (NS_FAILED(rv) || amtRead == 0)
       break;
-    if (!PREF_ParseBuf(&ps, fileBuffer, amtRead))
+    if (!PREF_ParseBuf(&ps, fileBuffer.get(), amtRead))
       rv2 = NS_ERROR_FILE_CORRUPTED;
     offset += amtRead;
     if (offset == fileSize) {
@@ -1177,7 +1215,7 @@ static nsresult pref_ReadPrefFromJar(nsZipArchive* jarReader, const char *name)
   NS_ENSURE_TRUE(manifest.Buffer(), NS_ERROR_NOT_AVAILABLE);
 
   PrefParseState ps;
-  PREF_InitParseState(&ps, PREF_ReaderCallback, nullptr);
+  PREF_InitParseState(&ps, PREF_ReaderCallback, ReportToConsole, nullptr);
   PREF_ParseBuf(&ps, manifest, manifest.Length());
   PREF_FinalizeParseState(&ps);
 
@@ -1270,9 +1308,7 @@ static nsresult pref_InitInitialObjects()
     "winpref.js"
 #elif defined(XP_UNIX)
     "unix.js"
-#if defined(VMS)
-    , "openvms.js"
-#elif defined(_AIX)
+#if defined(_AIX)
     , "aix.js"
 #endif
 #elif defined(XP_BEOS)
@@ -1315,7 +1351,7 @@ static nsresult pref_InitInitialObjects()
   // channel, telemetry is on by default, otherwise not. This is necessary
   // so that beta users who are testing final release builds don't flipflop
   // defaults.
-  if (Preferences::GetDefaultType(kTelemetryPref) == PREF_INVALID) {
+  if (Preferences::GetDefaultType(kTelemetryPref) == nsIPrefBranch::PREF_INVALID) {
     bool prerelease = false;
 #ifdef MOZ_TELEMETRY_ON_BY_DEFAULT
     prerelease = true;
@@ -1654,11 +1690,12 @@ Preferences::RemoveObservers(nsIObserver* aObserver,
 nsresult
 Preferences::RegisterCallback(PrefChangedFunc aCallback,
                               const char* aPref,
-                              void* aClosure)
+                              void* aClosure,
+                              MatchKind aMatchKind)
 {
   NS_ENSURE_TRUE(InitStaticMembers(), NS_ERROR_NOT_AVAILABLE);
 
-  ValueObserverHashKey hashKey(aPref, aCallback);
+  ValueObserverHashKey hashKey(aPref, aCallback, aMatchKind);
   RefPtr<ValueObserver> observer;
   gObserverTable->Get(&hashKey, getter_AddRefs(observer));
   if (observer) {
@@ -1666,7 +1703,7 @@ Preferences::RegisterCallback(PrefChangedFunc aCallback,
     return NS_OK;
   }
 
-  observer = new ValueObserver(aPref, aCallback);
+  observer = new ValueObserver(aPref, aCallback, aMatchKind);
   observer->AppendClosure(aClosure);
   nsresult rv = AddStrongObserver(observer, aPref);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -1678,9 +1715,10 @@ Preferences::RegisterCallback(PrefChangedFunc aCallback,
 nsresult
 Preferences::RegisterCallbackAndCall(PrefChangedFunc aCallback,
                                      const char* aPref,
-                                     void* aClosure)
+                                     void* aClosure,
+                                     MatchKind aMatchKind)
 {
-  nsresult rv = RegisterCallback(aCallback, aPref, aClosure);
+  nsresult rv = RegisterCallback(aCallback, aPref, aClosure, aMatchKind);
   if (NS_SUCCEEDED(rv)) {
     (*aCallback)(aPref, aClosure);
   }
@@ -1691,14 +1729,15 @@ Preferences::RegisterCallbackAndCall(PrefChangedFunc aCallback,
 nsresult
 Preferences::UnregisterCallback(PrefChangedFunc aCallback,
                                 const char* aPref,
-                                void* aClosure)
+                                void* aClosure,
+                                MatchKind aMatchKind)
 {
   if (!sPreferences && sShutdown) {
     return NS_OK; // Observers have been released automatically.
   }
   NS_ENSURE_TRUE(sPreferences, NS_ERROR_NOT_AVAILABLE);
 
-  ValueObserverHashKey hashKey(aPref, aCallback);
+  ValueObserverHashKey hashKey(aPref, aCallback, aMatchKind);
   RefPtr<ValueObserver> observer;
   gObserverTable->Get(&hashKey, getter_AddRefs(observer));
   if (!observer) {
@@ -1735,7 +1774,7 @@ Preferences::AddBoolVarCache(bool* aCache,
   data->cacheLocation = aCache;
   data->defaultValueBool = aDefault;
   gCacheData->AppendElement(data);
-  return RegisterCallback(BoolVarChanged, aPref, data);
+  return RegisterCallback(BoolVarChanged, aPref, data, ExactMatch);
 }
 
 static void IntVarChanged(const char* aPref, void* aClosure)
@@ -1760,7 +1799,7 @@ Preferences::AddIntVarCache(int32_t* aCache,
   data->cacheLocation = aCache;
   data->defaultValueInt = aDefault;
   gCacheData->AppendElement(data);
-  return RegisterCallback(IntVarChanged, aPref, data);
+  return RegisterCallback(IntVarChanged, aPref, data, ExactMatch);
 }
 
 static void UintVarChanged(const char* aPref, void* aClosure)
@@ -1785,9 +1824,43 @@ Preferences::AddUintVarCache(uint32_t* aCache,
   data->cacheLocation = aCache;
   data->defaultValueUint = aDefault;
   gCacheData->AppendElement(data);
-  return RegisterCallback(UintVarChanged, aPref, data);
+  return RegisterCallback(UintVarChanged, aPref, data, ExactMatch);
 }
 
+template <MemoryOrdering Order>
+static void AtomicUintVarChanged(const char* aPref, void* aClosure)
+{
+  CacheData* cache = static_cast<CacheData*>(aClosure);
+  *((Atomic<uint32_t, Order>*)cache->cacheLocation) =
+    Preferences::GetUint(aPref, cache->defaultValueUint);
+}
+
+template <MemoryOrdering Order>
+// static
+nsresult
+Preferences::AddAtomicUintVarCache(Atomic<uint32_t, Order>* aCache,
+                                   const char* aPref,
+                                   uint32_t aDefault)
+{
+  NS_ASSERTION(aCache, "aCache must not be NULL");
+#ifdef DEBUG
+  AssertNotAlreadyCached("uint", aPref, aCache);
+#endif
+  *aCache = Preferences::GetUint(aPref, aDefault);
+  CacheData* data = new CacheData();
+  data->cacheLocation = aCache;
+  data->defaultValueUint = aDefault;
+  gCacheData->AppendElement(data);
+  return RegisterCallback(AtomicUintVarChanged<Order>, aPref, data, ExactMatch);
+}
+
+// Since the definition of this template function is not in a header file,
+// we need to explicitly specify the instantiations that are required.
+// Currently only the order=Relaxed variant is needed.
+template
+nsresult Preferences::AddAtomicUintVarCache(Atomic<uint32_t,Relaxed>*,
+                                            const char*, uint32_t);
+
 static void FloatVarChanged(const char* aPref, void* aClosure)
 {
   CacheData* cache = static_cast<CacheData*>(aClosure);
@@ -1810,7 +1883,7 @@ Preferences::AddFloatVarCache(float* aCache,
   data->cacheLocation = aCache;
   data->defaultValueFloat = aDefault;
   gCacheData->AppendElement(data);
-  return RegisterCallback(FloatVarChanged, aPref, data);
+  return RegisterCallback(FloatVarChanged, aPref, data, ExactMatch);
 }
 
 // static
